//*********************************************************
// www.Ultimategameprogramming.com
// Tone Mapping effect shader for HDR lighting.
// Created by the Programming Ace.
//
// Input Variables used:
//    worldViewProj  - Holds the model view projection matrix.
//    fullTexture    - Holds the scene texture.
//    offset         - Texture coord offset.
//    exposure       - Exposure level of the image.
//
//*********************************************************


float4x4 worldViewProj : WorldViewProjection;
float2 offset;
float exposure;

texture fullTexture;


// This sampler struct is used to specifiy texture options.
sampler fullSampler = sampler_state
{
    Texture = (fullTexture);
};


// Vertex incoming data.
struct App_Input
{
   float3 vertexPos  : POSITION;
   float2 texture0   : TEXCOORD0;
};


// Vertex out going data.
struct Vs_Output
{
   float4 vertexPos  : POSITION;
   float2 texture0   : TEXCOORD0;
};


// Pixel out going data.
struct Ps_Output
{
	float4 color      : COLOR;
};


// The vertex shader...
Vs_Output ToneEffectVS(App_Input IN)
{
   Vs_Output vs_out;

   vs_out.vertexPos = mul(worldViewProj, float4(IN.vertexPos, 1));
   vs_out.texture0 = IN.texture0 + (offset * 0.5f);
   
   return vs_out;
}


// The pixel shader...
Ps_Output ToneEffectPS(Vs_Output IN)
{
   Ps_Output ps_out;

   float4 scenePass = tex2D(fullSampler, IN.texture0);

   scenePass = scenePass * exposure;
   ps_out.color = pow(scenePass, 0.55);

   return ps_out;
}


// Effect...
technique Tone
{
   pass Pass0
   {
      // Used by pixel shader.
      Sampler[0] = (fullSampler);

      // Here we will compile and set the vertex and pixel shader.
      VertexShader = compile vs_2_0 ToneEffectVS();
      PixelShader  = compile ps_2_0 ToneEffectPS();
   }
}